/**
 * An object to help with creating and using XMLHttpRequest.
 */
var RequestHelper = (function() {
    // Versions of MSXML XMLHTTP to try in IE browsers.
    var msXmlVersions = ["Msxml2.XMLHTTP.6.0", "Msxml2.XMLHTTP.3.0"];
    // The correct XMLHttpRequest function for this browser.
    var xhrFunction = null;

    // Determine what XMLHttpRequest object is available.
    if (window.XMLHttpRequest) {
        xhrFunction = function() { return new XMLHttpRequest(); };
    } else if (window.ActiveXObject) {
        // Try each XMLHTTP version until one works.
        for (var i = 0; i < msXmlVersions.length; i++) {
            xhrFunction = function() {
                return new ActiveXObject(msXmlVersions[i]);
            };

            try {
                xhrFunction();
            } catch (e) {
                continue;
            }

            break;
        }
    }

    // Return an object that defines the public API.
    return {
        /**
         * Creates and returns a new XMLHttpRequest instance.
         * Throws an error if this browser doesn't support
         * XMLHttpRequest or XMLHTTP.
         */
        createXHR: function() {
            var xhrObj = null;
            if (xhrFunction != null) {
                xhrObj = xhrFunction();
            }
            if (xhrObj == null) {
                // The browser doesn't support XMLHttpRequest
                // so throw an error.
                throw new Error("XMLHttpRequest not supported.");
            }
            return xhrObj;
        },

        /**
         * Returns the field name/value pairs from the given form,
         * concatenated into a single string, name and values
         * separated by '=' and pairs separated by '&', and encoded
         * according to the application/x-www-form-urlencoded
         * mime type.
         */
        encodeForm: function(form) {
            var encodedPairs = [];

            // Loop through the fields and encode the name/value pairs.
            for (var i = 0; i < form.elements.length; i++) {
                var field = form.elements[i];

                switch (field.type) {
                case "select-one":
                case "select-multiple":
                    // Get all the selected options.
                    for (var j = 0; j < field.options; j++) {
                        if (field.options[j].selected) {
                            encodedPairs.push(
                                encodePair(field.options[j].name,
                                    field.options[j].value));
                        }
                    }
                    break;

                case "checkbox":
                case "radio":
                    // Only used if selected.
                    if (!form.elements[i].checked) {
                        break;
                    }

                case "text":
                case "textarea":
                case "hidden":
                case "password":
                case "submit":
                case "reset":
                case "button":
                    encodedPairs.push(
                        encodePair(field.name, field.value));
                    break;
                }
            }

            // Concatenate the name/value pairs separate with '&'.
            return encodedPairs.join("&");
        },

        /**
         * Returns the given name/value pair, concatenated into
         * a single string separated by a '=', and encoded
         * according to the application/x-www-form-urlencoded
         * mime type.
         */
        encodePair: function(name, value) {
            // Use the encodeURIComponent() built in function.
            // Replace spaces encoded as %20 with +.
            return encodeURIComponent(name).replace(/%20/g, "+")
                + "=" + encodeURIComponent(value).replace(/%20/g, "+");
        },

        /**
         * Parses the given string of HTML and returns a
         * DocumentFragment containing the created Nodes.
         * If the given string contains no HTML tags,
         * then a TextNode is returned.
         */
        parseHTML: function(htmlString) {
            // Define a regular expression to match an HTML tag.
            var tagRegEx = /<(\w+)\s*(\w+\s*=\s*(?:".*?"|'.*?'|[^'">\s]+))*\s*\/?>([^<\/]*)(?:<\/\1\s*>)?/m;
            // Find the first HTML tag.
            var tagMatch = htmlString.match(tagRegEx);

            // If no HTML tags are present,
            // then return the given string as a TextNode.
            if (tagMatch == null) {
                return document.createTextNode(htmlString);
            }

            // Create a document to hold the parsed DOM elements.
            var docFragment = document.createDocumentFragment();
            var tagName = tagMatch[1];
            // Create a DOM object for the HTML tag.
            var tag = document.createElement(tagName);

            // If the tag has attributes, then add them to the object.
            if (tagMatch[2] != undefined) {
                // Define a regular expression to
                // break up the string of attributes.
                var attrRegEx = /\w+\s*=\s*(?:".*?"|'.*?'|[^'">\s]+)/gm;
                // Define a regular expression to break
                // an attribute into a name and value.
                var pairRegEx =
                    /(\w+)\s*=\s*(?:"(.*?)"|'(.*?)'|([^'">\s]+))/m;
                var attrMatch = tagMatch[2].match(attrRegEx);
    
                // Loop the the attributes and add them to the DOM object.
                for (var i = 0; i < attrMatch.length; i++) {
                    var pairMatch = attrMatch[i].match(pairRegEx);
                    var attrName = pairMatch[1];
                    
                    for (var j = 2; j < pairMatch.length; j++) {
                        if (pairMatch[j] != undefined) {
                            tag.setAttribute(attrName, pairMatch[j]);
                            break;
                        }
                    }
                }
            }

            // If the tag isn't closed then parse the tag content.
            if (tagMatch.length = 4) {
                var tagContent = tagMatch[3];

                // If the tag has content,
                // then parse it and append it to the tag.
                if (tagContent != undefined) {
                    tag.appendChild(this.parseHTML(tagContent));
                }
            }

            docFragment.appendChild(tag);
            var remainingStr = htmlString.substring(tagMatch.index
                + tagMatch[0].length);

            // Parse any remaining string.
            if (remainingStr.length > 0) {
                docFragment.appendChild(this.parseHTML(remainingStr));
            }

            return docFragment;
        }
    };
})();
